// =============================================================================
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) DIGITEO - 2011 - Cedric Delamarre
//
//  This file is distributed under the same license as the Scilab package.
// =============================================================================

// <-- ENGLISH IMPOSED -->

A=rand(2,4);
B=rand(2,4);

// ------------- Check errors cases -------------

// check arguments type.
ierr = execstr("gpuAdd(A,B,[B A])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Option syntax is [number,number].") then pause, end
ierr = execstr("gpuAdd(A,B,[0 A])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Inconsistent column/row dimensions.") then pause, end
ierr = execstr("gpuAdd(""toto"",B,[0 0])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Bad argument type of data input.") then pause, end
ierr = execstr("gpuAdd(A,""titi"",[0 0])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Bad argument type of data input.") then pause, end
ierr = execstr("gpuAdd(A,B,[""a"" ""tutu""])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Option syntax is [number,number].") then pause, end
ierr = execstr("gpuAdd(A,B,[45 156])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("First option argument must be 0 or 1 or 2.") then pause, end

C = sparse( [ 2  3  0  0  0;
              3  0  4  0  6; 
              0 -1 -3  2  0; 
              0  0  1  0  0; 
              0  4  2  0  1] );
Lup = umf_lufact(C);
ierr = execstr("gpuAdd(Lup,Lup)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Bad arguments type. Only variables created with GPU functions allowed.") then pause, end

// Check consistency between options and the number of output items.
ierr = execstr("[CPU GPU dA dB]=gpuAdd(A,B)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Too many output arguments.") then pause, end
ierr = execstr("gpuAdd(A,B,[2 12])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Too few output arguments.") then pause, end

// Don't duplicate the pointers of same data.
[CPU dA dB]=gpuAdd(A,B,[0 12]);
ierr = execstr("[CPU dC dD]=gpuAdd(dA,dB,[0 12])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("The first input argument is already a GPU variable.") then pause, end
if sum(gpuGetData(dA)-A) <> 0 then pause, end
ierr = execstr("gpuGetData(dC)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Undefined variable: dC") then pause, end

// ------------- Check the first option ------------- 

cpuRes=A+B;			// cpuRes = reference result

CPU=gpuAdd(A,B,[0 0]);		// same as CPU=gpuAdd(A,B)

if sum(cpuRes-CPU) > 1D-12, then pause, end

GPU=gpuAdd(A,B,[1 0]);		// result is stored only on the device.
if sum(gpuGetData(GPU) - cpuRes) > 1D-12 then pause, end //gpuGetData(GPU), transfer the result on the host (Device -> Host).

[CPU GPU]=gpuAdd(A,B,[2 0]);	// result is stored on the device (in "GPU") and transfer on the host ( in "CPU"). 
if sum(gpuGetData(GPU) - cpuRes) > 1D-12 then pause, end
if sum(cpuRes-CPU) > 1D-12 then pause, end

// ------------- Check the second option ------------- 

[CPU dA]=gpuAdd(A,B,[0 1]);	// dA is a pointer which contain A data on the Device.
if sum(gpuGetData(dA)-A) <> 0 then pause, end

[CPU dB]=gpuAdd(A,B,[0 2]);	// dB is a pointer which contain B data on the Device.
if sum(gpuGetData(dB)-B) <> 0 then pause, end

dA=gpuFree(dA);			// Free GPU pointer before reuse it in Scilab.
if dA <> 0 then pause, end
dB=gpuFree(dB);
if dB <> 0 then pause, end
[CPU dA dB]=gpuAdd(A,B,[0 12]); 
if sum(gpuGetData(dB)-B) <> 0 then pause, end
if sum(gpuGetData(dA)-A) <> 0 then pause, end

gpuFree(dA);
clear dA;
gpuFree(dB);			// Free GPU pointers at the end of program.
clear dB;


// ------------- Real/Complex ------------- 

ca=rand(4,6)+%i*rand(4,6);
cb=2*rand(4,6)+%i*3*rand(4,6);
c=rand(4,6);
d=2*rand(4,6);

if sum(gpuAdd(2,3) - (2+3)) <> 0 then pause, end
if sum(gpuAdd(c,3) - (c+3)) <> 0 then pause, end
if sum(gpuAdd(2,d) - (2+d)) <> 0 then pause, end
if sum(gpuAdd(c,d) - (c+d)) <> 0 then pause, end

if sum(gpuAdd(2*%i,3*%i) - (2*%i+3*%i)) <> 0 then pause, end
if sum(gpuAdd(ca,3*%i) - (ca+3*%i)) <> 0 then pause, end
if sum(gpuAdd(2*%i,cb) - (2*%i+cb)) <> 0 then pause, end
if sum(gpuAdd(ca,cb) - (ca+cb)) <> 0 then pause, end

if sum(gpuAdd(2,3*%i) - (2+3*%i)) <> 0 then pause, end
if sum(gpuAdd(c,3*%i) - (c+3*%i)) <> 0 then pause, end
if sum(gpuAdd(2,cb) - (2+cb)) <> 0 then pause, end
if sum(gpuAdd(c,cb) - (c+cb)) <> 0 then pause, end

if sum(gpuAdd(2*%i,3) - (2*%i+3)) <> 0 then pause, end
if sum(gpuAdd(ca,3) - (ca+3)) <> 0 then pause, end
if sum(gpuAdd(2*%i,d) - (2*%i+d)) <> 0 then pause, end
if sum(gpuAdd(ca,d) - (ca+d)) <> 0 then pause, end


// ------------- some tests ------------- 
stacksize('max');
a = rand(256,256);
ierr = execstr("gpuAdd(a,a)","errcatch");
if ierr == 999 then pause,end;

a = rand(3000,3000);
ierr = execstr("gpuAdd(a,a)","errcatch");
if ierr == 999 then pause,end;

