// =============================================================================
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) DIGITEO - 2011 - Cedric Delamarre
//
//  This file is distributed under the same license as the Scilab package.
// =============================================================================

// <-- ENGLISH IMPOSED -->

A=rand(3,2);
Z=rand(3,4)+%i*rand(3,4);

// ------------- Check errors cases -------------

// check arguments type.
ierr = execstr("gpuFFT(A,[0 A])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Inconsistent column/row dimensions.") then pause, end
ierr = execstr("gpuFFT(""toto"",[0 0])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Bad argument type.") then pause, end
ierr = execstr("gpuFFT(A,[""a"" ""tutu""])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Option syntax is [number,number].") then pause, end
ierr = execstr("gpuFFT(A,[45 156])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("First option argument must be 0 or 1 or 2.") then pause, end
ierr = execstr("gpuFFT(A,[0 2])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Second option argument must be 0 or 1.") then pause, end

C = sparse( [ 2  3  0  0  0;
              3  0  4  0  6; 
              0 -1 -3  2  0; 
              0  0  1  0  0; 
              0  4  2  0  1] );
Lup = umf_lufact(C);
ierr = execstr("gpuFFT(Lup)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Bad arguments type. Only variables created with GPU functions allowed.") then pause, end

// Check consistency between options and the number of output items.
ierr = execstr("[CPU GPU dA]=gpuFFT(A)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Too many output arguments.") then pause, end
ierr = execstr("gpuFFT(A,[2 1])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Too few output arguments.") then pause, end

// Don't duplicate the pointers of same data.
[CPU dA]=gpuFFT(A,[0 1]);
ierr = execstr("[CPU dB]=gpuFFT(dA,[0 1])","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("The first input argument is already a GPU variable.") then pause, end
if sum(gpuGetData(dA)-A) <> 0 then pause, end
ierr = execstr("gpuGetData(dB)","errcatch");
msg = lasterror();
if ierr == 0 or msg <> gettext("Undefined variable: dB") then pause, end

// ------------- Check the first option ------------- 

cpuRes=fft(A);			// cpuRes = reference result

CPU=gpuFFT(A,[0 0]);		// same as CPU=gpuMult(A,B)
if sum(cpuRes-CPU) <> 0 then pause, end

GPU=gpuFFT(A,[1 0]);		// result is stored only on the device.
if sum(gpuGetData(GPU) - cpuRes) <> 0 then pause, end //gpuGetData(GPU), transfer the result on the host (Device -> Host).

[CPU GPU]=gpuFFT(A,[2 0]);	// result is stored on the device (in "GPU") and transfer on the host ( in "CPU").
if sum(gpuGetData(GPU) - cpuRes) <> 0 then pause, end
if sum(cpuRes-CPU) <> 0 then pause, end

// ------------- Check the second option ------------- 

[CPU dA]=gpuFFT(A,[0 1]);	// dA is a pointer which contain A data on the Device.
if sum(gpuGetData(dA)-A) <> 0 then pause, end
gpuFree(dA);
clear dA

// ------------- Real/Complex ------------- 

s=sum(gpuFFT(Z) - fft(Z));
if abs(real(s))+abs(imag(s)) > 1D-14 then pause, end
if sum(gpuFFT(A) - fft(A)) <> 0 then pause, end

if sum(gpuFFT(5+2*%i) - fft(5+2*%i)) <> 0 then pause, end

// ------------- some tests -------------

//stacksize('max');
//a = rand(4000,4000) + rand(4000,4000) * %i;
//ierr = execstr("gpuFFT(a)","errcatch");
//if ierr <> 2 then pause,end;
// 2 == Cuda error : The API call failed because it was unable to allocate enough memory to perform the requested operation.

